unit uFormularioAutomatico;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  maskedit, EditBtn, Grids, ExtCtrls, ComCtrls, Buttons, SelectTypeUnit,
  strutils, ucosa, uvisordetabla, uStringGridWithControls, TypInfo, contnrs, DifferencesTreeUnit,
  uFormularioBase, uRegistroDeEditores;

type

  //////////////////////////////////////////////////////////////////////////////
  // SUB-CONTROLES                                                            //
  //////////////////////////////////////////////////////////////////////////////

  { TIUCosaButton }

  TIUCosaButton=class(TButton)
    procedure ButtonClick(Sender: TObject);
  private
    DataCellParent:TDataCellOfCosa;
    DataTableChildName:string;
    DataRowChildId:integer;
  public
    constructor Create(xParentControl:TWinControl; xDataCellParent:TDataCellOfCosa; xDataTableChildName: string; xDataRowChildId: integer);
  end;

  { TICosaChildButton }

  TICosaChildButton=class(TIUCosaButton)
    procedure ButtonClick(Sender: TObject);
  private
    xItemListClaseDeCosa:TClaseDeCosa;
  public
    constructor Create(xParentControl:TWinControl; xDataCellParent:TDataCellOfCosa; xItemListClaseDeCosa: TClaseDeCosa);
  end;

  { TVDT_DefaultEditor }

  TVDT_DefaultEditor=class(TStringGridWithControls)

    DataTable:TDataTableOfCosa;

    constructor Create( AOwner: TComponent; Name: String);

    procedure Lupa_OnClick(nidRec: string; kFila: integer); override;
    procedure Clonar_OnClick(nidRec: string; kFila: integer); override;
    procedure Borrar_OnClick(nidRec: string; kFila: integer); override;

    function GetDataRow(kRow: Integer):TDataRowOfCosa;

    procedure Populate_DataTable ( dt: TDataTableOfCosa );

  end;

  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  // CONTROLES                                                                //
  //////////////////////////////////////////////////////////////////////////////

  { TEditorControl }

  TEditorControl=class(TPanel)
    procedure LocalControlOnEnter(Sender:TObject);
    procedure LocalControlOnExit(Sender: TObject);

  public

    OldSval:string;

    Point:TPoint;

    ALabel:TLabel;
    DataColumn:TDataColumnOfCosa;
    DataCell:TDataCellOfCosa;

    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); virtual;
    procedure Place(xTop, xLeft: integer); virtual;
  end;

  { TEditorTEditControl }

  TEditorTEditControl=class(TEditorControl)
  private
    AEdit:TEdit;
  public
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorTButtonControl }

  TEditorTButtonControl=class(TEditorControl)
  public
    AButton:TIUCosaButton;
    constructor Create(AOwner: TComponent; xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
      xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorTCheckBoxControl }

  TEditorTCheckBoxControl=class(TEditorControl)
  public
    ACheckBox:TCheckBox;
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;

  end;

  { TEditorTDateEditControl }

  TEditorTDateEditControl=class(TEditorControl)
  private
    ADateEdit:TDateEdit;
  public
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorTMaskEditControl }

  TEditorTMaskEditControl=class(TEditorControl)
    procedure LocalAEditOnExit(Sender: TObject);
  private
    AEdit:TEdit;
    AMaskEdit:TMaskEdit;
  public
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorTMemoControl }

  TEditorTMemoControl=class(TEditorControl)
  private
    AMemo:TMemo;
  public
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa); override;
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorDataGridControl }

  TEditorDataGridControl=class(TEditorControl)
  private
    ItemListClaseDeCosa:TClaseDeCosa;
  public
    AVT_DefaultEditor: TVDT_DefaultEditor;
    AButton: TIUCosaButton;
    constructor Create(AName:string; AOwner: TFormularioBase; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa; xItemListClaseDeCosa:TClaseDeCosa);
    procedure Place(xTop, xLeft: integer); override;
    procedure RefreshTabla;
  end;

  { TEditorTComboBoxControl }

  TEditorTComboBoxControl=class(TEditorControl)
    procedure LocalComboBoxOnChange(Sender: TObject);
  public
    AComboBox:TComboBox;
    ClaseDeCosa:TClaseDeCosa;
    SelectedItem:string;

    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa);
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TDAOfStringTComboBoxControl }

  TDAOfStringTComboBoxControl=class(TEditorControl)
    procedure LocalComboBoxOnChange(Sender: TObject);
  public
    AComboBox:TComboBox;
    ClaseDeCosa:TClaseDeCosa;
    SelectedItem:string;

    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa);
    procedure Place(xTop, xLeft: integer); override;
  end;

  { TEditorTOpenDialogControl }

  TEditorTOpenDialogControl=class(TEditorControl)
    procedure LocalAButtonOnClick(Sender: TObject);
  private
    AEdit:TEdit;
    AButton:TButton;
  public
    constructor Create(AOwner: TComponent; xParentControl:TWinControl; xDataColumn: TDataColumnOfCosa; xDataCell:TDataCellOfCosa);
    procedure Place(xTop, xLeft: integer); override;
  end;

  //////////////////////////////////////////////////////////////////////////////

  { TFormularioAutomatico }

  TFormularioAutomatico = class(TFormularioBase)
    Button1: TButton;
    CancelButton: TButton;
    ScrollBox: TScrollBox;
    SaveButton:TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    { private declarations }
    dcl:TDataColumnListOfCosa;
    Point:TPoint;
  protected
    ControlList:TList;
  public
    { public declarations }

    constructor Create(xDataSet: TDataStore; xParentForm:TFormularioBase; xTitle:string);


    procedure CreateControlsForOneDataRow(DataTable: TDataTableOfCosa);
    procedure CreateControlsForMultipleDataRow(DataTAble: TDataTableOfCosa);

    procedure PlaceControls();

    procedure SetControls(); virtual;
    procedure SetEventHandlers(); virtual;
    procedure RunEventHandlers(); virtual;

    function GetControlByDataColumnName(AColumnName: string): TEditorControl;
    function GetIndexOfControl(AColumnName:string):integer;

  end;

  procedure PrintDataSet(DataSet:TDataStore);

var
  COSA_WITH_CUSTOMIZEDFORM_LIST:TFPHashList;

implementation

uses
  CosaEditorUnit;

var
  xcount:integer;

{$R *.lfm}

{ TVDT_DefaultEditor }

constructor TVDT_DefaultEditor.Create(AOwner: TComponent; Name: String);
begin
  Inherited Create(AOwner, Name);
end;

procedure TVDT_DefaultEditor.Lupa_OnClick(nidRec: string; kFila: integer);
var
  ADataSet:TDataStore;

  frmCurrent, frmChild: TFormularioBase;
  ctrlParent:TEditorDataGridControl;

  CosaStrId: String;
  Value: String;
  DataRow: TDataVRowOfCosa;
  b: String;
  siud: TSIUD;
begin

  frmCurrent:=TFormularioAutomatico(self.Owner);

  if assigned(frmCurrent.DataSet.GetDataTableByName('dt_TFuenteConstante')) then
    b:=frmCurrent.DataSet.GetDataTableByName('dt_TFuenteConstante').DataRowList[0].DataCellList[
      frmCurrent.DataSet.GetDataTableByName('dt_TFuenteConstante').GetIndexOfDataColumnByName('nombresdebornes_publicados')
    ].sval;

  ctrlParent:=TEditorDataGridControl(self.Parent.Parent);

  DataRow:=TDataVRowOfCosa(self.GetDataRow(kFila));
  ADataSet:= TDataStore.Create(frmCurrent.DataSet);

  Value:= DataRow.OriginalDataTableName+'.'+IntToStr(DataRow.OriginalDataRowId);

  ADataSet:=frmCurrent.DataSet.PrepareUpdate(ctrlParent.DataCell.DataRowOwner.DataTableOwner.TableName,
    ctrlParent.DataColumn.ColumnName, ctrlParent.DataCell.DataRowOwner._RowId, Value, IsChild(TListaDeCosas, ctrlParent.DataColumn.DataType.ClaseDeCosa));

  CosaStrId:=copy(DataRow.OriginalDataTableName,4,Length(DataRow.OriginalDataTableName));

  frmChild:= OpenForm (frmCurrent ,
                       CosaStrId,
                       ADataSet,
                       frmCurrent.Caption+' >> '+ctrlParent.DataColumn.ColumnName+'['+name+']');

  if (frmChild.ShowModal() = mrOk) then
    if frmChild is TFormularioAutomatico and frmChild.hayCambios then
      frmCurrent.DataSet.Merge(frmChild.DataSet)
    else
      frmChild.SaveToDataStore();

end;

procedure TVDT_DefaultEditor.Borrar_OnClick(nidRec: string; kFila: integer);
var
  ADataSet:TDataStore;
  frmCurrent: TFormularioAutomatico;

  DataRowParent: TDataRowOfCosa;
  DataRowChild: TDataVRowOfCosa;
  ctrlParent: TEditorDataGridControl;
  Value: String;
  DataTableParentName: String;
begin

  frmCurrent:=TFormularioAutomatico(self.Owner);
  ctrlParent:=TEditorDataGridControl(self.Parent.Parent);

  DataRowParent:=ctrlParent.DataCell.DataRowOwner;
  DataTableParentName:=ctrlParent.DataCell.DataRowOwner.DataTableOwner.TableName;
  DataRowChild:=TDataVRowOfCosa(self.GetDataRow(kFila));

  Value:= DataRowChild.OriginalDataTableName+'.'+IntToStr(DataRowChild.OriginalDataRowId);
  ADataSet:=frmCurrent.DataSet.PrepareDelete(DataTableParentName, ctrlParent.DataColumn.ColumnName,
    DataRowParent._RowId, Value);
  if MessageDlg('BORRAR !!!!', TMsgDlgType.mtConfirmation, [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], 0) = mrYes then
  begin
    frmCurrent.DataSet.Merge(ADataSet);
//    dropFila( filas[kFila] );
  end
  else
    ADataSet.Free;

end;

function TVDT_DefaultEditor.GetDataRow(kRow: Integer): TDataRowOfCosa;
begin
//  self.Rows[kRow].
end;

procedure TVDT_DefaultEditor.Clonar_OnClick(nidRec: string; kFila: integer);
begin
  inherited;
end;

procedure TVDT_DefaultEditor.Populate_DataTable(dt: TDataTableOfCosa);
var
  i,j: Integer;
  DataColumn: TDataColumnOfCosa;
  DataCell: TDataCellOfCosa;
  DataRow: TDataRowOfCosa;

  encabezado: array of string;
  tipoColumnas: array of TTipoColumna;
begin

  if dt=nil then exit;

  //self.DataTable:=dt.Clone();

  //Encabezado
  SetLength(encabezado, dt.DataColumnListCount+1);
  SetLength(tipoColumnas, dt.DataColumnListCount+1);

  self.ColCount:=dt.DataColumnListCount+1;
  self.RowCount:=dt.DataRowListCount+1;

  for i:=0 to dt.DataColumnListCount-1 do
  begin
    DataColumn:=dt.DataColumnList[i];
    encabezado[i]:=DataColumn.ColumnName;
    tipoColumnas[i]:=TC_Texto;
  end;
  encabezado[dt.DataColumnListCount]:='';
  tipoColumnas[dt.DataColumnListCount]:=TC_BitBtns;

  self.SetEncabezados(encabezado, tipoColumnas);

  //Filas
  for i:=0 to dt.DataRowListCount-1 do
  begin
    DataRow:= dt.DataRowList[i];
    for j:= 0 to dt.DataColumnListCount-1 do
      wrText(j, i+1, DataRow.DataCellList[j].sval);

    wrBitBtns(j+1, i+1, [iid_Lupa, iid_Editar, iid_Borrar], []);
  end;
end;


{ TButtonSelectType }

procedure TICosaChildButton.ButtonClick(Sender: TObject);
var
  ok: Boolean;
  frmCurrent: TFormularioBase;
  ctrlParent: TEditorDataGridControl;
  xSelectTypeForm: TSelectTypeForm;
  ADataVTable: TDataVTableOfCosa;
  ADataSet: TDataStore;
  siud: TSIUD;
  frmChild: TFormularioBase;
   ClassName:string;
begin
  ok:=false;
  ctrlParent:=TEditorDataGridControl(self.Parent);
  xSelectTypeForm:=TSelectTypeForm.Create(self.Owner);

  ADataVTable:=TDataVTableOfCosa(DATA_STORE.GetDataTableByName('dvt_'+self.xItemListClaseDeCosa.ClassName));
  if (Assigned(ADataVTable)) then
  begin
    xSelectTypeForm.UpdateListBox(ADataVTable);
    if xSelectTypeForm.ShowModal = mrOK then
    begin
      ClassName:=xSelectTypeForm.ClaseDeCosaChild.ClassName;
      self.DataTableChildName:='dt_'+ClassName;
      ok:=true;
    end;
  end
  else
  begin
    ClassName:=self.xItemListClaseDeCosa.ClassName;
    self.DataTableChildName:='dt_'+ClassName;
    ok:=true;
  end;

  if (ok) then
  begin
    frmCurrent:=TFormularioBase(self.Owner);

    ADataSet:=frmCurrent.DataSet.PrepareInsert(ctrlParent.DataCell.DataRowOwner.DataTableOwner.TableName,
      ctrlParent.DataColumn.ColumnName, ctrlParent.DataCell.DataRowOwner._RowId,
        DataTableChildName);

    siud:=TSIUD.TINSERT;

    frmChild:= OpenForm(frmCurrent, ClassName,
                                  ADataSet,
                                  frmCurrent.Caption+' >> '+TEditorControl(self.Parent).DataColumn.ColumnName+' -> Nuevo '+ClassName);
    if (frmChild.ShowModal() = mrOK) then
    begin
      frmCurrent.DataSet.Merge(frmChild.DataSet);
      if siud = TSIUD.TINSERT then
        self.DataRowChildId:=ctrlParent.DataCell.DataRowOwner._RowId;

    end;

    //ctrlParent.RefreshTabla

  end;
end;

constructor TICosaChildButton.Create(xParentControl: TWinControl;
  xDataCellParent: TDataCellOfCosa; xItemListClaseDeCosa: TClaseDeCosa);
begin
  inherited Create(xParentControl, xDataCellParent, '', -1);
  self.xItemListClaseDeCosa:=xItemListClaseDeCosa;
  self.OnClick:=self.ButtonClick;
end;

////////////////////////////////////////////////////////////////////////////////

{ TIUCosaButton }

constructor TIUCosaButton.Create(xParentControl: TWinControl;
  xDataCellParent: TDataCellOfCosa; xDataTableChildName: string;
  xDataRowChildId: integer);
begin
  inherited Create(xParentControl.Owner);
  self.Parent:=xParentControl;
  self.DataCellParent:=xDataCellParent;
  self.DataTableChildName:=xDataTableChildName;
  self.DataRowChildId:=xDataRowChildId;
  self.OnClick:=self.ButtonClick;
end;

procedure TIUCosaButton.ButtonClick(Sender: TObject);
var
  ADataSet:TDataStore;
  frmCurrent, frmChild: TFormularioBase;

  DataRowParent, DataRowChild: TDataRowOfCosa;

  ctrlParent: TEditorDataGridControl;
  Value: String;
  DataTableParentName: String;
  siud: TSIUD;
  split: TStringList;
  CosaStrId: String;
  b: String;
begin

  frmCurrent:=TFormularioAutomatico(self.Owner);

  ctrlParent:=TEditorDataGridControl(self.Parent);

  DataRowParent:=ctrlParent.DataCell.DataRowOwner;
  DataTableParentName:=ctrlParent.DataCell.DataRowOwner.DataTableOwner.TableName;

  Value:=self.DataCellParent.sval;

  if (self.DataRowChildId<>-1) then
  begin
    siud:=TSIUD.TUPDATE;

    b:=ctrlParent.DataColumn.DataType.ClaseDeCosa.ClassName;

    ADataSet:=frmCurrent.DataSet.PrepareUpdate(DataTableParentName, ctrlParent.DataColumn.ColumnName,
      DataRowParent._RowId, Value, IsChild(TListaDeCosas, ctrlParent.DataColumn.DataType.ClaseDeCosa));
  end
  else
  begin
    siud:=TSIUD.TINSERT;
    ADataSet:=frmCurrent.DataSet.PrepareInsert(ctrlParent.DataCell.DataRowOwner.DataTableOwner.TableName,
      ctrlParent.DataColumn.ColumnName, ctrlParent.DataCell.DataRowOwner._RowId,
      DataTableChildName, IsChild(TListaDeCosas, ctrlParent.DataColumn.DataType.ClaseDeCosa));
  end;

  CosaStrId:=Copy(self.DataTableChildName, 4, Length(self.DataTableChildName)-1);

  frmChild:= OpenForm(frmCurrent, CosaStrId,
                          ADataSet,
                          frmCurrent.Caption+' >> '+TEditorControl(self.Parent).DataColumn.ColumnName);

  if (frmChild.ShowModal() = mrOK) and (frmChild.hayCambios)then
  begin
    frmCurrent.DataSet.Merge(frmChild.DataSet);

    if siud = TSIUD.TINSERT then
    begin
      split:=TStringList.Create;
      ExtractStrings(['.'], [], PChar(DataCellParent.sval), split);
      self.DataRowChildId:= StrToInt ( split[1] );
    end;
  end;

  //Lo debería hacer el frmChild al cerrar
  ADataSet.Free();

end;

////////////////////////////////////////////////////////////////////////////////

{ TDAOfStringTComboBoxControl }

procedure TDAOfStringTComboBoxControl.LocalComboBoxOnChange(Sender: TObject);
begin

end;

constructor TDAOfStringTComboBoxControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
begin

  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.AComboBox:=TComboBox.Create(xParentControl.Owner);
  self.AComboBox.Parent:=self;
  self.AComboBox.Name:='cb_'+self.DataColumn.ColumnName;
  self.AComboBox.Caption:='';
  self.AComboBox.Width:=200;
  self.AComboBox.OnExit:=self.LocalControlOnExit;
  self.AComboBox.OnChange:=self.LocalComboBoxOnChange;

  if (self.DataCell.sval<>'') then
  begin
    self.AComboBox.ItemIndex:=self.AComboBox.Items.IndexOf(self.DataCell.sval);
  end;
end;

procedure TDAOfStringTComboBoxControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AComboBox.Top:=0;
  self.AComboBox.Left:=250;
end;

{ TEditorTOpenDialogControl }

procedure TEditorTOpenDialogControl.LocalAButtonOnClick(Sender: TObject);
var
  AOpenDialog:TOpenDialog;
  filename: String;
  a: string;
begin

  AOpenDialog:=TOpenDialog.Create(self.Owner);
  //AOpenDialog.Filter := 'Sala SimSEE|*.ese';
  if AOpenDialog.Execute then
  begin
    filename:=AOpenDialog.Filename;

    a:=self.AEdit.Text;

    self.AEdit.Text:=filename;
    self.AEdit.SetFocus();

    AOpenDialog.Free;
  end;
end;

constructor TEditorTOpenDialogControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
begin

  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.AEdit:=TEdit.Create(xParentControl.Owner);
  self.AEdit.Parent:=self;
  self.AEdit.Name:='e_'+DataColumn.ColumnName;
  self.AEdit.Caption:='';
  self.AEdit.OnEnter:=self.OnEnter;
  self.AEdit.OnExit:=self.LocalControlOnExit;
  self.AEdit.Width:=400;

  self.AButton:=TButton.Create(xParentControl.Owner);
  self.AButton.Parent:=self;
  self.AButton.Name:='b_'+DataColumn.ColumnName;
  self.AButton.Caption:=' ... ';
  self.AButton.OnClick:=self.LocalAButtonOnClick;
  self.AButton.Width:=30;

  if (xDataCell<>nil) and (xDataCell.sval<>'') then
    self.AEdit.Text:=xDataCell.sval;

end;

procedure TEditorTOpenDialogControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AEdit.Top:=0;
  self.AEdit.Left:=250;

  self.AButton.Top:=0;
  self.AButton.Left:=250+400+5;
end;

{ TEditorTComboBoxControl }

procedure TEditorTComboBoxControl.LocalComboBoxOnChange(Sender: TObject);
var
  ADataSet: TDataStore;
  frmCurrent: TFormularioAutomatico;
  frmChild: TFormularioAutomatico;

  SelectTypeForm:TSelectTypeForm;

  TheTableName:string;
  ADataVTable:TDataVTableOfCosa;
  ok: Boolean;
begin
  ok:=false;
  if (self.AComboBox.ItemIndex > 0) then
  begin
    self.SelectedItem:=self.AComboBox.Items[self.AComboBox.ItemIndex];
  end
  else
  begin
    frmCurrent:=TFormularioAutomatico(self.Owner);
    ADataSet:=TDataStore.Create(frmCurrent.DataSet);

    SelectTypeForm:=TSelectTypeForm.Create(self.Owner);
    ADataVTable:=TDataVTableOfCosa(DATA_STORE.GetDataTableByName('dvt_'+self.ClaseDeCosa.ClassName));
    if (Assigned(ADataVTable)) then
    begin
      SelectTypeForm.UpdateListBox(ADataVTable);
      if SelectTypeForm.ShowModal = mrOK then
      begin
        TheTableName:='dt_'+SelectTypeForm.ClaseDeCosaChild.ClassName;
        ok:=true;
      end
    end
    else
    begin
      TheTableName:='dt_'+self.ClaseDeCosa.ClassName;
      ok:=true;
    end;

  if (ok) then
  begin
      frmCurrent.DataSet.Query('*', TheTableName, 'AAA', '999', ADataSet);

      frmChild:=TFormularioAutomatico.Create(ADataSet, frmCurrent, frmCurrent.Caption+' >> '+self.DataColumn.ColumnName);
      if frmChild.ShowModal() = mrOK then
      begin
        self.SelectedItem:=frmChild.DataSet.DataTableList[0].DataRowList[0].DataCellList[frmChild.DataSet.DataTableList[0].GetIndexOfDataColumnByName('nombre')].sval;
        self.AComboBox.Items.Add(self.SelectedItem);
        self.AComboBox.ItemIndex:=self.AComboBox.Items.Count-1;
        frmCurrent.DataSet.Merge(frmChild.DataSet);
      end;
    end;
  end;
end;

constructor TEditorTComboBoxControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
var
  DataSet: TDataStore;
  DataView: TDataTableOfCosa;
  frmCurrent: TFormularioAutomatico;
  i: Integer;

  split: TStringList;
  a: Integer;
  ADataCell: TDataCellOfCosa;
  sval: String;
begin

  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.ClaseDeCosa:=xDataColumn.DataType.ClaseDeCosa;

  self.AComboBox:=TComboBox.Create(xParentControl.Owner);
  self.AComboBox.Parent:=self;
  self.AComboBox.Name:='cb_'+self.DataColumn.ColumnName;
  self.AComboBox.Caption:='';
  self.AComboBox.Width:=200;
  self.AComboBox.OnExit:=self.LocalControlOnExit;
  self.AComboBox.OnChange:=self.LocalComboBoxOnChange;

  frmCurrent:=TFormularioAutomatico(self.Owner);

  AComboBox.items.Add('<Agregar Nuevo>');

//  DataSet:=TDataStore.Create(frmCurrent.DataSet);
    DataView := frmCurrent.DataSet.View('nombre', 'dvt_'+self.ClaseDeCosa.ClassName);

//frmCurrent.DataSet.Query('*', 'dvt_'+self.ClaseDeCosa.ClassName, DataSet);
//  DataView:=TDataVTableOfCosa(DataSet.DataTableList[0]);

  for i:=0 to DataView.DataRowListCount-1 do
  begin
    ADataCell:=DataView.DataRowList[i].DataCellList[0];
    AComboBox.items.AddObject(DataView.DataRowList[i].DataCellList[0].sval, ADataCell);
  end;

  sval:=self.DataCell.sval;
  if ((sval<>'') and (sval<>'<?.?>')) then
  begin
    split:=TStringList.Create;
    ExtractStrings(['<', '.', '>'], [], PChar(sval), split);
    self.SelectedItem:=split[1];
    self.AComboBox.ItemIndex:=self.AComboBox.Items.IndexOf(self.SelectedItem);
  end;
end;

procedure TEditorTComboBoxControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AComboBox.Top:=0;
  self.AComboBox.Left:=250;
end;

{ TEditorDataGridControl }

constructor TEditorDataGridControl.Create(AName: string;
  AOwner: TFormularioBase; xParentControl: TWinControl;
  xDataColumn: TDataColumnOfCosa; xDataCell: TDataCellOfCosa;
  xItemListClaseDeCosa: TClaseDeCosa);
var
  split: TStringList;
  DataCellSval:string;
  DataColumnName:string;
  DataTableName:string;

  aDataSet:TDataStore;
  aDataTable:TDataTableOfCosa;
  aDataColumn:TDataColumnOfCosa;
  aDataRow:TDataRowOfCosa;
  aDataCell:TDataCellOfCosa;

  z,i,j:integer;

  aSValSplit: TStringList;

  aDataRowChildId: Integer;
  aDataTableChildName: String;

  aDataSetChild:TDataStore;
  aDataTableChild:TDataTableOfCosa;
  aDataColumnChild:TDataColumnOfCosa;
  aDataRowChild:TDataRowOfCosa;
  aDataCellChild:TDataCellOfCosa;

  ic,jc:integer;

  frmCurrent: TFormularioAutomatico;

  q: Integer;

begin

  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.ItemListClaseDeCosa:=xItemListClaseDeCosa;

  //Obtengo el formilario padre del control
  frmCurrent:=TFormularioAutomatico(self.Owner);

  self.aVT_DefaultEditor := TVDT_DefaultEditor.Create(AOwner, 'vt_'+AName );
  self.aVT_DefaultEditor.Parent:=self;
  self.AVT_DefaultEditor.Width:=400;
  self.AVT_DefaultEditor.Height:=220;

  self.RefreshTabla;

  self.AButton:=TICosaChildButton.Create(xParentControl, xDataCell, self.ItemListClaseDeCosa);
  self.AButton.Parent:=self;
  self.AButton.Name:='btn_'+AName;
  self.AButton.Caption:=' + ';
  self.AButton.Width:=30;

end;

procedure TEditorDataGridControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);


  self.AButton.Top:=0;
  self.AButton.Left:=250+400+5;
end;

procedure TEditorDataGridControl.RefreshTabla;
var
  frmCurrent: TFormularioAutomatico;
  split: TStringList;
  DataTableName: String;
  DataColumnName: String;
  DataCellSval: String;
  aDataSet: TDataStore;
  aDataTable: TDataTableOfCosa;
  aDataColumn: TDataColumnOfCosa;
  aDataRow: TDataRowOfCosa;
  aDataCell1, aDataCell2: TDataCellOfCosa;
  aSValSplit: TStringList;
  aDataTableChildName: String;
  aDataRowChildId: Integer;
  j: Integer;
  aDataSetChild:TDataStore;
begin

  //Obtengo el formilario padre del control
  frmCurrent:=TFormularioAutomatico(self.Owner);

  if (self.DataCell.sval<>'') then
  begin
    split:=TStringList.Create;
    try
      ExtractStrings(['.'], [], PChar(self.DataCell.sval), split);
      DataTableName:=split[0];
      DataColumnName:=split[1];
      DataCellSval:=split[2];
    finally
      split.Free;
    end;
      aDataSet:=TDataStore.Create(frmCurrent.DataSet);
      frmCurrent.DataSet.Query('*', DataTableName, DataColumnName, DataCellSval, aDataSet);
      aDataSetChild:=TDataStore.Create(aDataSet);


      if (aDataSet<>nil) and (aDataSet.DataTableListCount=1) then
      begin
        aDataTable:=aDataSet.DataTableList[0];

        //ChildId
        aDataColumn:=aDataTable.DataColumnList[1];
        for j:=0 to aDataTable.DataRowListCount-1 do
        begin
          aDataRow:=aDataTable.DataRowList[j];
          if aDataRow.SIUD<>TSIUD.TDELETE then
          begin
            aDataCell1:=aDataRow.DataCellList[1];
            split:=TStringList.Create;
            try
              ExtractStrings(['.'], [], PChar(aDataCell1.sval), split);
              aDataTableChildName:=split[0];
              aDataRowChildId:= StrToInt (split[1]);
            finally
              split.Free;
            end;
            frmCurrent.DataSet.Query('*', aDataTableChildName, aDataRowChildId, aDataSetChild);
          end;
        end;
      end;

    PrintDataSet(aDataSetChild);
    self.aVT_DefaultEditor.Populate_DataTable(aDataSetChild.View('nombre'));
  end;
end;

{ TEditorTMaskEditControl }

procedure TEditorTMaskEditControl.LocalAEditOnExit(Sender: TObject);
var
  xlength: string;
  mask:string;
  i: Integer;
begin
  xlength:=self.AEdit.Text;
  for i:=1 to StrToInt(xlength) do
    mask:=mask+'#.##, ';
  mask:=Copy(mask,1,Length(mask)-2);
  mask:=mask+';1;_';
  self.AMaskEdit.EditMask:=mask;
end;

constructor TEditorTMaskEditControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
var
  i:integer;
  sval:string;
  xlength:string;
  xelements:string;
  split:TStringList;
  mask:string;
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.AEdit:=TEdit.Create(self);
  self.AEdit.Parent:=self;
  self.AEdit.Name:='e_'+DataColumn.ColumnName;
  self.AEdit.Caption:='';
  self.AEdit.Width:=30;
  //self.AEdit.OnExit:=LocalControlOnExit;

  sval:=self.DataCell.sval;
  sval:=Copy(sval,2,Length(sval)-2);

  split := TStringList.Create;
  try
    ExtractStrings(['|'], [], PChar(sval), split);
    xlength:=split[0];
    xelements:=split[1];
  finally
    split.Free;
  end;

  self.AEdit.Text:=xlength;

  self.AMaskEdit:=TMaskEdit.Create(self);
  self.AMaskEdit.Parent:=self;
  self.AMaskEdit.Name:='me_'+DataColumn.ColumnName;
  self.AMaskEdit.Caption:='';
  self.AMaskEdit.Width:=100;

  //self.LocalAEditOnExit(nil);

  self.AMaskEdit.OnExit:=self.LocalControlOnExit;
  if (self.DataCell<>nil) and (self.DataCell.sval<>'') then
    self.AMaskEdit.Text:=xelements;
end;

procedure TEditorTMaskEditControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AEdit.Top:=0;
  self.AEdit.Left:=250;

  self.AMaskEdit.Top:=0;
  self.AMaskEdit.Left:=250+40;
end;

{ TEditorTMemoControl }

constructor TEditorTMemoControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
var
  i:integer;
  sval:string;
  xlength:string;
  xelements:string;
  split:TStringList;
  mask:string;
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.AMemo:=TMemo.Create(xParentControl.Owner);
  self.AMemo.Parent:=self;
  self.AMemo.Name:='m_'+DataColumn.ColumnName;
  self.AMemo.Caption:='';
  self.AMemo.Width:=400;
  self.AMemo.Height:=50;
  self.AMemo.ScrollBars:=ssVertical;
  self.AMemo.OnEnter:=self.LocalControlOnEnter;
  self.AMemo.OnExit:=self.LocalControlOnExit;
  if (self.DataCell<>nil) and (self.DataCell.sval<>'') then
  begin

    sval:=self.DataCell.sval;
    sval:=Copy(sval,2,Length(sval)-2);

    split := TStringList.Create;
    try
      ExtractStrings(['|'], [], PChar(sval), split);
      xlength:=split[0];
      xelements:=split[1];
      xelements:=Trim(xelements);
      self.AMemo.Text:=xelements;
    finally
      split.Free;
    end;
  end;
end;

procedure TEditorTMemoControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AMemo.Top:=0;
  self.AMemo.Left:=250;

end;

{ TEditorTDateEditControl }

constructor TEditorTDateEditControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.ADateEdit:=TDateEdit.Create(self);
  self.ADateEdit.Parent:=self;
  self.ADateEdit.Name:='de_'+DataColumn.ColumnName;
  self.ADateEdit.Caption:='';
  self.ADateEdit.OnExit:=self.LocalControlOnExit;
  if (xDataCell<>nil) and (xDataCell.sval<>'') then
    self.ADateEdit.Text:=xDataCell.sval;
end;

procedure TEditorTDateEditControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.ADateEdit.Top:=0;
  self.ADateEdit.Left:=250;
end;

{ TEditorTCheckBoxControl }

constructor TEditorTCheckBoxControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.ACheckBox:=TCheckBox.Create(self);
  self.ACheckBox.Parent:=self;
  self.ACheckBox.Name:='cb_'+DataColumn.ColumnName;
  self.ACheckBox.Caption:='';
  self.ACheckBox.OnExit:=self.LocalControlOnExit;
  if (xDataCell<>nil) and (xDataCell.sval<>'') then
    self.ACheckBox.Checked:=StrToBool(xDataCell.sval);

end;

procedure TEditorTCheckBoxControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.ACheckBox.Top:=0;
  self.ACheckBox.Left:=250;
end;

{ TEditorTButtonControl }

constructor TEditorTButtonControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
var
  split: TStringList;
  DataTableChileName:string;
  DataRowChildId:integer;
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);
  if (xDataCell<>nil) and (xDataCell.sval<>'')then
  begin
    split:=TStringList.Create;
    try
      ExtractStrings(['.'], [], PChar(xDataCell.sval), split);
      DataTableChileName:=split[0];
      DataRowChildId:=StrToInt(split[1]);
    finally
      split.Free;
    end;
  end
  else
  begin
    DataTableChileName:='dt_'+xDataColumn.DataType.ClaseDeCosa.ClassName;
    DataRowChildId:=-1;
  end;
  self.AButton:=TIUCosaButton.Create(self, xDataCell, DataTableChileName, DataRowChildId);
  self.AButton.Parent:=self;
  self.AButton.Name:='b_'+DataColumn.ColumnName;
  self.AButton.Caption:='...';
  self.AButton.Width:=30;
end;

procedure TEditorTButtonControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);

  self.AButton.Top:=0;
  self.AButton.Left:=250;
end;

{ TEditorTEditControl }

constructor TEditorTEditControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
begin
  inherited Create(AOwner, xParentControl, xDataColumn, xDataCell);

  self.AEdit:=TEdit.Create(self);
  self.AEdit.Parent:=self;
  self.AEdit.Name:='e_'+DataColumn.ColumnName;
  self.AEdit.Caption:='';
  self.AEdit.OnEnter:=self.OnEnter;
  self.AEdit.OnExit:=self.LocalControlOnExit;
  if (xDataCell<>nil) and (xDataCell.sval<>'') then
    self.AEdit.Text:=xDataCell.sval;

  if DataColumn.DataType is TDataTypeStringOfCosa then
    self.AEdit.Width:=400;
end;

procedure TEditorTEditControl.Place(xTop, xLeft: integer);
begin
  inherited Place(xTop, xLeft);
  self.AEdit.Top:=0;
  self.AEdit.Left:=250;
end;

{ TEditorControl }

constructor TEditorControl.Create(AOwner: TComponent;
  xParentControl: TWinControl; xDataColumn: TDataColumnOfCosa;
  xDataCell: TDataCellOfCosa);
var
  a: Integer;
  b:string;
begin

  inherited Create(AOwner);
  self.Parent:=xParentControl;

  self.BevelInner:=bvNone;
  self.BevelOuter:=bvNone;

  if self is TEditorDataGridControl then
  begin
    self.Height:=230;
    self.Point.y:=230;
  end
  else if self is TEditorTMemoControl then
  begin
    self.Height:=50;
    self.Point.y:=50;
  end
  else
  begin
    self.Height:=24;
    self.Point.y:=24;
  end;

  self.Width:=700;

  a:=self.Height;
  b:=xDataColumn.ColumnName;
  self.DataColumn:=xDataColumn;
  self.DataCell:=xDataCell;

  a:=self.DataCell.DataRowOwner._RowId;

  self.ALabel:=TLabel.Create(self);
  self.ALabel.Parent:=self;
  self.ALabel.Name:='l_'+DataColumn.ColumnName;
  self.ALabel.Caption:=DataColumn.ColumnName+':';

end;

procedure TEditorControl.Place(xTop, xLeft: integer);
begin
  self.Top:=xTop;
  self.Left:=xLeft;

  self.ALabel.Top:=0;
  self.ALabel.Left:=0;
end;

procedure TEditorControl.LocalControlOnEnter(Sender: TObject);
var
  AControl:TControl;
  sval:string;
begin
  if Sender is TEdit then begin
    sval:=TEdit(Sender).Text;
    TEdit(Sender).SelStart:=0;
    TEdit(Sender).SelLength:=maxLongInt;
    TEdit(Sender).SelectAll();

    self.OldSval:=sval;

  end
end;

procedure TEditorControl.LocalControlOnExit(Sender: TObject);
var
  sval:string;
  AControl:TControl;
  DataCell:TDataCellOfCosa;
begin

  AControl:=nil;

  if Sender is TEdit then begin
    AControl:=TEdit(Sender);
    sval:=TEdit(Sender).Text
  end
  else if Sender is TCheckBox then begin
    AControl:=TCheckBox(Sender);
    sval:=BoolToStr(TCheckBox(Sender).Checked)
  end
  else if Sender is TDateEdit then begin
    AControl:=TDateEdit(Sender);
    sval:=DateTimeToStr(TDateEdit(Sender).Date)
  end
  else if Sender is TComboBox then begin
    AControl:=TComboBox(Sender);
    if TEditorControl(AControl.Parent) is TEditorTComboBoxControl then
    begin
      if (TComboBox(Sender).Items[TComboBox(Sender).ItemIndex] <> '') then
        sval:='<'+TEditorTComboBoxControl(self).ClaseDeCosa.ClassName+'.'+TComboBox(Sender).Items[TComboBox(Sender).ItemIndex]+'>'
      else
        sval:='<?.?>';
    end
    else if TEditorControl(AControl.Parent) is TDAOfStringTComboBoxControl then
    begin
      if (TComboBox(Sender).Items.Count>0) then
        sval:=TComboBox(Sender).Items[TComboBox(Sender).ItemIndex];
    end;
  end;

  if (sval<>self.OldSval) then
  begin

    TFormularioAutomatico(self.Owner).hayCambios:=true;

    if (AControl<>nil) then
    begin
      DataCell:=TEditorControl(AControl.Parent).DataCell;
      if (DataCell<>nil) then
        DataCell.sval:=sval;
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

{ TFormularioAutomatico }


procedure TFormularioAutomatico.PlaceControls();
var
  AControl:TEditorControl;
  i:integer;
  Border: TPanel;

begin

  //self.Top:=self.Parent.Top+50;
  //self.Left:=self.Parent.Left+50;

  for i:=0 to self.ControlList.Count-1 do
  begin
    AControl:=TEditorControl(self.ControlList.Items[i]);
    AControl.Place(Point.y, Point.x);

    Point.y:=Point.y+AControl.Point.y+10;
  end;

  //Para agregar un espacio al final...
  Border:=TPanel.Create(self);
  Border.Parent:=self.ScrollBox;
  Border.Top:=Point.y;
  Border.Left:=Point.x;
  Border.Height:=20;
  Border.BevelInner:=bvNone;
  Border.BevelOuter:=bvNone;
end;

procedure TFormularioAutomatico.SetControls;
begin

end;

procedure TFormularioAutomatico.SetEventHandlers;
begin

end;

procedure TFormularioAutomatico.RunEventHandlers;
begin

end;

function TFormularioAutomatico.GetControlByDataColumnName(AColumnName: string
  ): TEditorControl;
var
  i: Integer;
begin
  Result:=nil;
  i:=self.GetIndexOfControl('BornePreciosPorCombustible');
  if (i>-1) then
  begin
    Result:=TEditorControl(self.ControlList.Items[i]);
  end;
end;

function TFormularioAutomatico.GetIndexOfControl(AColumnName: string):integer;
var
  i: Integer;
  AControl: TEditorControl;
begin
  Result:=-1;
  for i:=0 to self.ControlList.Count-1 do
  begin
    AControl:=TEditorControl(self.ControlList.Items[i]);
    if (LowerCase(AControl.DataColumn.ColumnName)=LowerCase(AColumnName)) then
    begin
      Result:=i;
      break;
    end;
  end;
end;


procedure TFormularioAutomatico.FormCreate(Sender: TObject);
var
  z: Integer;
  DataTable: TDataTableOfCosa;
  i: Integer;
  sval: String;
  DataColumn: TDataColumnOfCosa;
  Control: TEditorTCheckBoxControl;
  j: Integer;
  o:integer;
  AButton:TButton;
  FormParent: TFormularioAutomatico;
begin

  Point.x:=10;
  Point.y:=10;

  self.ControlList:=TList.Create();

  for z:=0 to self.DataSet.DataTableListCount-1 do
  begin
    DataTable:=self.DataSet.DataTableList[z];
    if DataTable.Visible then
    begin
      if DataTable.DataRowListCount <= 1 then
      begin
        self.CreateControlsForOneDataRow(DataTable);
      end
      else
      begin
        self.CreateControlsForMultipleDataRow(DataTable);
      end;
    end;
  end;
  self.SaveButton.OnClick:=SaveButtonClick;


end;

procedure TFormularioAutomatico.Button1Click(Sender: TObject);
begin
  PrintDataSet(self.DataSet);

end;

procedure TFormularioAutomatico.FormShow(Sender: TObject);
begin
  //self.Text:=self.ClaseDeCosa.ClassName;
  self.Top:=0;
  self.Left:=0;
end;

procedure TFormularioAutomatico.SaveButtonClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TFormularioAutomatico.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrClose;
end;

constructor TFormularioAutomatico.Create(xDataSet: TDataStore;
  xParentForm: TFormularioBase; xTitle: string);
begin

  inherited Create(xParentForm);
  self.DataSet:=xDataSet;
  self.ParentForm:=xParentForm;
  self.Caption:=xTitle;
  self.DataSet.SetName:=xTitle;
  self.hayCambios:=false;

end;

procedure TFormularioAutomatico.CreateControlsForOneDataRow(DataTable: TDataTableOfCosa);
var
  z,i,j: Integer;
  DataColumn: TDataColumnOfCosa;
  sval: String;
  Control: TEditorControl;
  DataRow: TDataRowOfCosa;
  DataCell:TDataCellOfCosa;
  a:integer;
  b:string;
begin
  DataRow:=DataTable.DataRowList[0];
  for i:=0 to DataTable.DataColumnListCount-1 do
  begin
    DataColumn:=DataTable.DataColumnList[i];

    if (DataTable.TableName='TFichaCentralAguasArriba') then
      a:=1;

    if (LowerCase(DataColumn.ColumnName)='lpdforzamientos') then
    begin
      a:=1;
      WriteLn(DataColumn.DataType.ClassName);
    end;

    DataCell:=DataRow.DataCellList[i];

    a:=DataCell.DataRowOwner._RowId;

    sval:=DataCell.sval;

    if (DataColumn.DataType is TDataTypeBooleanOfCosa) then
      Control:=TEditorTCheckBoxControl.Create(self, self.ScrollBox, DataColumn, DataCell)

    else if (DataColumn.DataType is TDataTypeCardinalOfCosa) or
            (DataColumn.DataType is TDataTypeDoubleOfCosa)   or
            (DataColumn.DataType is TDataTypeIntegerOfCosa)  or
            (DataColumn.DataType is TDataTypeStringOfCosa)   then
      Control:=TEditorTEditControl.Create(self, self.ScrollBox, DataColumn, DataCell)

    else if (DataColumn.DataType is TDataTypeDAOfBooleanOfCosa) or
            (DataColumn.DataType is TDataTypeDAOfDoubleOfCosa)  or
            (DataColumn.DataType is TDataTypeDAOfIntegerOfCosa) or
            (DataColumn.DataType is TDataTypeDAOfStringOfCosa)  then
      Control:=TEditorTMemoControl.Create(self, self.ScrollBox, DataColumn, DataCell)

    else if (DataColumn.DataType is TDataTypeDateOfCosa) then
      Control:=TEditorTDateEditControl.Create(self, self.ScrollBox, DataColumn, DataCell)

    else if (DataColumn.DataType is TDataTypeDataRowOfCosa) then
    begin
      if (DataColumn.ColumnName='lpd')then
        a:=1;
      if (DataColumn.DataType.ClaseDeCosa<>nil) then
        Control:=TEditorTButtonControl.Create(self, self.ScrollBox, DataColumn, DataCell)
    end

    else if (DataColumn.DataType is TDataTypeListOfCosa) then
    begin
      Control:=TEditorDataGridControl.Create('Grid',self, self.ScrollBox, DataColumn, DataCell, DataTable.GetDataColumnList().ItemListClaseDeCosa);
    end

    else if (DataColumn.DataType is TDataTypeReferenciaOfCosa) then
    begin
      if (DataColumn.DataType.ClaseDeCosa<>nil) then
        Control:=TEditorTComboBoxControl.Create(self, self.ScrollBox, DataColumn, DataCell);
    end

    else if (DataColumn.DataType is TDataTypeArchiRefOfCosa) then
    begin
      Control:=TEditorTOpenDialogControl.Create(self, self.ScrollBox, DataColumn, DataCell);
    end
    ;

    self.ControlList.Add(Control);
  end;

  self.SetControls();
  self.SetEventHandlers();
  self.RunEventHandlers();

  self.PlaceControls();

end;

procedure TFormularioAutomatico.CreateControlsForMultipleDataRow(DataTAble: TDataTableOfCosa);
var
  vt:TVDT_DefaultEditor;
begin

  //vt := TVDT_DefaultEditor.Create(ScrollBox, 'vt_'+DataTAble.TableName, nil);
  //vt.Populate_DataTable;

end;


procedure PrintDataSet(DataSet:TDataStore);
var
  z: Integer;
  j: Integer;
  i: Integer;
begin
  WriteLn();
  WriteLn('---');
  WriteLn('SetName = ' + DataSet.SetName);
  for z:=0 to DataSet.DataTableListCount-1 do
  begin
    WriteLn('   ' + 'TableName = ' + DataSet.DataTableList[z].TableName);
    for j:=0 to DataSet.DataTableList[z].DataRowListCount-1 do
    begin
      WriteLn('      ' + 'RowId = ' + IntToStr(DataSet.DataTableList[z].DataRowList[j]._RowId) + ' | SIUD = ' + GetEnumName(TypeInfo(TSIUD), ord(DataSet.DataTableList[z].DataRowList[j].SIUD)));
      for i:=0 to DataSet.DataTableList[z].DataColumnListCount-1 do
      begin
        WriteLn('         ' + DataSet.DataTableList[z].DataColumnList[i].ColumnName+' = '+DataSet.DataTableList[z].DataRowList[j].DataCellList[i].sval);
      end;
    end;
  end;
  WriteLn('---');
  WriteLn();
end;

end.

